@@ -1,6 +1,6 @@
 package Juju;
-$Juju::VERSION = '1.5.1';
-# ABSTRACT: Perl bindings for Juju
+$Juju::VERSION = '1.1';
+# ABSTRACT: Non-blocking Perl bindings for https://juju.ubuntu.com/
 
 
 use strict;
@@ -18,15 +18,15 @@ __END__
 
 =head1 NAME
 
-Juju - Perl bindings for Juju
+Juju - Non-blocking Perl bindings for https://juju.ubuntu.com/
 
 =head1 VERSION
 
-version 1.5.1
+version 1.1
 
 =head1 DESCRIPTION
 
-Perl non-blocking/blocking bindings for Juju. See
+Pure perl non-blocking bindings for Juju. See
 L<Juju::Manual::Quickstart> for more information.
 
 =head1 AUTHOR
@@ -41,18 +41,6 @@ This is free software, licensed under:
 
   The MIT (X11) License
 
-=head1 SEE ALSO
-
-Please see those modules/websites for more information related to this module.
-
-=over 4
-
-=item *
-
-L<https://juju.ubuntu.com|https://juju.ubuntu.com>
-
-=back
-
 =head1 DISCLAIMER OF WARRANTY
 
 BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
