@@ -1,15 +1,13 @@
 package Juju::Environment;
-$Juju::Environment::VERSION = '1.5.1';
+$Juju::Environment::VERSION = '1.1';
 # ABSTRACT: Exposed juju api environment
 
 
 use strict;
 use warnings;
+use HTTP::Tiny;
 use JSON::PP;
 use YAML::Tiny qw(Dump);
-use Data::Validate::Type qw(:boolean_tests);
-use Params::Validate qw(:all);
-use Juju::Util;
 use parent 'Juju::RPC';
 
 
@@ -17,16 +15,32 @@ use Class::Tiny qw(password is_authenticated), {
     endpoint => sub {'wss://localhost:17070'},
     username => sub {'user-admin'},
     Jobs     => sub {
-        {   HostUnits     => 'JobHostUnits',
+        +{  HostUnits     => 'JobHostUnits',
             ManageEnviron => 'JobManageEnviron',
             ManageState   => 'JobManageSate'
         };
-    },
-    util   => Juju::Util->new
+    }
 };
 
 
 
+sub query_cs {
+    my ($self,   $charm)  = @_;
+    my ($series, $_charm) = $charm =~ /^(precise|trusty)\/(\w+)/i;
+    my $cs_url = 'https://manage.jujucharms.com/api/3/charm';
+    if (!$series) {
+        $series = 'trusty';
+        $_charm = $charm;
+    }
+
+    my $composed_url = sprintf("%s/%s/%s", $cs_url, $series, $_charm);
+    my $res = HTTP::Tiny->new->get($composed_url);
+    die "Unable to query charm store\n" unless $res->{success};
+    return decode_json($res->{content});
+}
+
+
+
 sub _prepare_constraints {
     my ($self, $constraints) = @_;
     foreach my $key (keys %{$constraints}) {
@@ -40,31 +54,28 @@ sub _prepare_constraints {
 
 sub login {
     my $self = shift;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
-
-    my $params = {
-        "Type"      => "Admin",
-        "Request"   => "Login",
-        "RequestId" => $self->request_id,
-        "Params"    => {
-            "AuthTag"  => $self->username,
-            "Password" => $self->password
+    $self->create_connection unless $self->is_connected;
+    $self->call(
+        {   "Type"      => "Admin",
+            "Request"   => "Login",
+            "RequestId" => $self->request_id,
+            "Params"    => {
+                "AuthTag"  => $self->username,
+                "Password" => $self->password
+            }
+        },
+        sub {
+            $self->is_authenticated(1);
         }
-    };
-
-    # block
-    my $res = $self->call($params);
-    die $res->{Error} if defined($res->{Error});
-    $self->is_authenticated(1)
-      unless !defined($res->{Response}->{EnvironTag});
+    );
 }
 
 
 
-
 sub reconnect {
     my $self = shift;
     $self->close;
+    $self->create_connection;
     $self->login;
     $self->request_id = 1;
 }
@@ -321,37 +332,19 @@ sub environment_set {
 
 
 sub add_machine {
-    my $self = shift;
-    my ($series, $constraints, $machine_spec, $parent_id, $container_type) =
-      validate_pos(
-        @_,
-        {default => 'trusty'},
-        {type    => HASHREF, default => +{}},
-        0, 0, 0
-      );
+    my ($self, $series, $constraints, $machine_spec, $parent_id,
+        $container_type)
+      = @_;
+    my $cb     = ref $_[-1] eq 'CODE' ? pop : undef;
 
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
     my $params = {
         "Series"        => $series,
-        "Jobs"          => [$self->Jobs->{HostUnits}],
-        "ParentId"      => "",
-        "ContainerType" => ""
+        "Constraints"   => $self->_prepare_constraints($constraints),
+        "ContainerType" => $container_type,
+        "ParentId"      => $parent_id,
+        "Jobs"          => $self->Jobs->{HostUnits},
     };
 
-    # validate constraints
-    if (defined($constraints) and is_hashref($constraints)) {
-        $params->{Constraints} = $self->_prepare_constraints($constraints);
-    }
-
-    # if we're here then assume constraints is good and we can check the
-    # rest of the arguments
-    if (defined($machine_spec)) {
-        die "Cant specify machine spec with container_type/parent_id"
-          if $parent_id or $container_type;
-        ($params->{ParentId}, $params->{ContainerType}) = split /:/,
-          $machine_spec;
-    }
-
     return $self->add_machines([$params]) unless $cb;
     return $self->add_machines([$params], $cb);
 }
@@ -450,73 +443,45 @@ sub remove_relation {
 
 
 sub deploy {
-    my $self = shift;
-    my ($charm, $service_name) =
-      validate_pos(@_, {type => SCALAR}, {type => SCALAR});
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+    my ($self, $service_name, $charm_url, $num_units, $config_yaml,
+        $constraints, $machine_spec)
+      = @_;
+    my $cb     = ref $_[-1] eq 'CODE' ? pop : undef;
 
-    # parse additional arguments
-    my ($num_units, $config_yaml, $constraints, $machine_spec) = @_;
-
-    my $params = {
-        Type    => "Client",
-        Request => "ServiceDeploy",
-        Params  => {ServiceName => $service_name}
-    };
-
-    # Check for series format
-    my (@charm_args) = $charm =~ /(\w+)\/(\w+)/i;
-    my $_charm_url = undef;
-    if (scalar @charm_args == 2) {
-        $_charm_url = $self->util->query_cs($charm_args[1], $charm_args[0]);
-    }
-    else {
-        $_charm_url = $self->util->query_cs($charm);
-    }
-
-    $params->{Params}->{CharmUrl} = $_charm_url->{charm}->{url};
+    my $params = {ServiceName => $service_name};
     $num_units = 1 unless $num_units;
-    $params->{Params}->{NumUnits} = $num_units;
-    $params->{Params}->{ConfigYAML} =
-      defined($config_yaml) ? $config_yaml : "";
-
-    if (defined($constraints) and is_hashref($constraints)) {
-        $params->{Params}->{Constraints} =
-          $self->_prepare_constraints($constraints);
+    $params->{NumUnits}   = $num_units;
+    $params->{ConfigYAML} = $config_yaml;
+    my $svc_constraints;
+    if ($constraints) {
+        $params->{Constraints} = $self->_prepare_constraints($constraints);
     }
     if ($machine_spec) {
-        $params->{Params}->{ToMachineSpec} = "$machine_spec";
+        $params->{ToMachineSpec} = $machine_spec;
     }
-
-    # block
-    return $self->call($params) unless $cb;
-
-    # non-block
-    return $self->call($params, $cb);
+    $self->call(
+        {   "Type"    => "Client",
+            "Request" => "ServiceDeploy",
+            "Params"  => $params
+        }
+    );
 }
 
 
-sub service_set {
-    my $self = shift;
-    my ($service_name, $config) =
-      validate_pos(@_, {type => SCALAR}, {type => HASHREF, optional => 1});
+sub set_config {
+    my ($self, $service_name, $config) = @_;
+    my $cb     = ref $_[-1] eq 'CODE' ? pop : undef;
 
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
-
-    my $params = {
-        "Type"    => "Client",
-        "Request" => "ServiceSet",
-        "Params"  => {
-            "ServiceName" => $service_name,
-            "Options"     => $config
+    die "Not a hash" unless ref $config eq 'HASH';
+    return $self->call(
+        {   "Type"    => "Client",
+            "Request" => "ServiceSet",
+            "Params"  => {
+                "ServiceName" => $service_name,
+                "Options"     => $config
+            }
         }
-    };
-
-    # block
-    return $self->call($params) unless $cb;
-
-    # non-block
-    return $self->call($params, $cb);
+    );
 }
 
 
@@ -524,47 +489,37 @@ sub unset_config {
     my ($self, $service_name, $config_keys) = @_;
     my $cb     = ref $_[-1] eq 'CODE' ? pop : undef;
 
-    my $params = 
+    return $self->call(
         {   "Type"    => "Client",
             "Request" => "ServiceUnset",
             "Params"  => {
                 "ServiceName" => $service_name,
                 "Options"     => $config_keys
             }
-        };
-
-    # block
-    return $self->call($params) unless $cb;
-
-    # non-block
-    return $self->call($params, $cb);
+        }
+    );
 }
 
 
 sub set_charm {
     my ($self, $service_name, $charm_url, $force) = @_;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+    my $cb     = ref $_[-1] eq 'CODE' ? pop : undef;
 
     $force = 0 unless $force;
-    my $params = {
-        "Type"    => "Client",
-        "Request" => "ServiceSetCharm",
-        "Params"  => {
-            "ServiceName" => $service_name,
-            "CharmUrl"    => $charm_url,
-            "Force"       => $force
+    $self->call(
+        {   "Type"    => "Client",
+            "Request" => "ServiceSetCharm",
+            "Params"  => {
+                "ServiceName" => $service_name,
+                "CharmUrl"    => $charm_url,
+                "Force"       => $force
+            }
         }
-    };
-
-    # block
-    return $self->call($params) unless $cb;
-
-    # non-block
-    return $self->call($params, $cb);
+    );
 }
 
 
-sub service_get {
+sub get_service {
     my ($self, $service_name) = @_;
     my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
     my $params = {
@@ -585,7 +540,7 @@ sub get_config {
     my ($self, $service_name) = @_;
     my $cb     = ref $_[-1] eq 'CODE' ? pop : undef;
 
-    my $svc = $self->service_get($service_name);
+    my $svc = $self->get_service($service_name);
     return $svc->{Config} unless $cb;
     return $cb->($svc->{Config});
 }
@@ -924,7 +879,7 @@ Juju::Environment - Exposed juju api environment
 
 =head1 VERSION
 
-version 1.5.1
+version 1.1
 
 =head1 SYNOPSIS
 
@@ -960,29 +915,25 @@ Supported juju jobs
 
 =head1 METHODS
 
-=head2 _prepare_constraints
+=head2 query_cs ($charm)
+
+helper for querying charm store for charm details
+
+=head2 _prepare_constraints ($constraints)
 
 Makes sure cpu-cores, cpu-power, mem are integers
 
-B<Params>
+C<constraints> - hash of service constraints
 
-=over 4
-
-=item *
-
-C<constraints>
-
-hash of service constraints
-
-=back
+B<Returns> - an updated constraint hash with any integers set properly.
 
 =head2 login
 
-Login to juju, will die on a failed login attempt.
+Login to juju
 
 =head2 reconnect
 
-Reconnects to API server in case of timeout, this also resets the RequestId.
+Reconnects to API server in case of timeout
 
 =head2 environment_info
 
@@ -992,53 +943,18 @@ Return Juju Environment information
 
 Environment UUID from client connection
 
-=head2 environment_unset
+=head2 environment_unset ($items)
 
 Environment UUID from client connection
 
-B<Params>
-
-=over 4
-
-=item *
-
-C<items>
-
-=back
-
-=head2 find_tools
+=head2 find_tools ($major_version, $minor_version, $series, $arch)
 
 Returns list containing all tools matching specified parameters
 
-B<Params>
-
-=over 4
-
-=item *
-
-C<major_verison>
-
-major version int
-
-=item *
-
-C<minor_verison>
-
-minor version int
-
-=item *
-
-C<series>
-
-Distribution series (eg, trusty)
-
-=item *
-
-C<arch>
-
-architecture
-
-=back
+C<major_verison> - major version int
+C<minor_verison> - minor version int
+C<series> - Distribution series (eg, trusty)
+C<arch> - architecture
 
 =head2 agent_version
 
@@ -1060,145 +976,59 @@ Returns network hostports for each api server
 
 Returns watcher
 
-=head2 get_watched_tasks
+=head2 get_watched_tasks ($watcher_id)
 
 List of all watches for Id
 
-B<Params>
-
-=over 4
-
-C<watcher_id>
-
-=back
-
-=head2 add_charm
+=head2 add_charm ($charm_url)
 
 Add charm
 
-B<Params>
+C<charm_url> - url of charm
 
-=over 4
-
-=item *
-
-C<charm_url>
-
-url of charm
-
-=back
-
-=head2 get_charm
+=head2 get_charm ($charm_url)
 
 Get charm
 
-B<Params>
-
-=over 4
-
-=item *
-
-C<charm_url>
-
-url of charm
-
-=back
+C<charm_url> - url of charm
 
 =head2 get_environment_constraints
 
 Get environment constraints
 
-=head2 set_environment_constraints
+=head2 set_environment_constraints ($constraints)
 
 Set environment constraints
 
-B<Params>
-
-=over 4
-
-=item *
-
-C<constraints>
-
-environment constraints
-
-=back
+C<constraints> - environment constraints
 
 =head2 environment_get
 
 Returns all environment settings
 
-=head2 environment_set
+=head2 environment_set ($config)
 
-Sets the given key-value pairs in the environment.
+C<config> - Config parameters
 
-B<Params>
-
-=over 4
-
-=item *
-
-C<config>
-
-Config parameters
-
-=back
-
-=head2 add_machine
+=head2 add_machine ($series, $constraints, $machine_spec, $parent_id, $container_type)
 
 Allocate new machine from the iaas provider (i.e. MAAS)
 
-B<Params>
+C<series> - OS series (i.e precise)
 
-=over 4
+C<constraints> - machine constraints
 
-=item *
+C<machine_spec> - specific machine
 
-C<series>
+C<parent_id> - id of parent
 
-OS series (i.e precise)
+C<container_type> - kvm or lxc container type
 
-=item *
-
-C<constraints>
-
-machine constraints
-
-=item *
-
-C<machine_spec>
-
-specific machine
-
-=item *
-
-C<parent_id>
-
-id of parent
-
-=item *
-
-C<container_type>
-
-kvm or lxc container type
-
-=back
-
-=head2 add_machines
+=head2 add_machines ($machines)
 
 Add multiple machines from iaas provider
 
-B<Params>
-
-=over 4
-
-=item *
-
-C<machines>
-
-List of machines
-
-=back
+C<machines> - List of machines
 
 =head2 destroy_environment
 
@@ -1208,437 +1038,122 @@ Destroys Juju environment
 
 Destroy machines
 
-B<Params>
-
-=over 4
-
-=item *
-
-C<machine_ids>
-
-List of machines
-
-=item *
-
-C<force>
-
-Force destroy
-
-=back
-
 =head2 provisioning_script
 
 Not implemented
 
-=head2 add_relation
+=head2 add_relation ($endpoint_a, $endpoint_b)
 
 Sets a relation between units
 
-B<Params>
-
-=over 4
-
-=item *
-
-C<endpoint_a>
-
-First unit endpoint
-
-=item *
-
-C<endpoint_b>
-
-Second unit endpoint
-
-=back
-
-=head2 remove_relation
+=head2 remove_relation ($endpoint_a, $endpoint_b)
 
 Removes relation between endpoints
 
-B<Params>
-
-=over 4
-
-=item *
-
-C<endpoint_a>
-
-First unit endpoint
-
-=item *
-
-C<endpoint_b>
-
-Second unit endpoint
-
-=back
-
-=head2 deploy
+=head2 deploy ($service_name, $charm_url, $num_units, $config_yaml, $constraints, $machine_spec)
 
 Deploys a charm to service
 
-B<Params>
-
-=over 4
-
-=item *
-
-C<charm>
-
-charm to deploy, can be in the format of B<series/charm> if needing to specify a different series
-
-=item *
-
-C<service_name>
-
-name of service to set. can be same name as charm, however, recommended to pick something unique and identifiable.
-
-=item *
-
-C<num_units>
-
-(optional) number of service units
-
-=item *
-
-C<config_yaml>
-
-(optional) A YAML formatted string of charm options
-
-=item *
-
-C<constraints>
-
-(optional) Machine hardware constraints
-
-=item *
-
-C<machine_spec>
-
-(optional) Machine specification
-
-=back
-
-More information on deploying can be found by running C<juju help deploy>.
-
-=head2 service_set
+=head2 set_config ($service_name, $config)
 
 Set's configuration parameters for unit
 
-B<Params>
+C<service_name> - name of service (ie. blog)
 
-=over 4
+C<config> - hash of config parameters
 
-=item *
-
-C<service_name>
-
-name of service (ie. blog)
-
-=item *
-
-C<config>
-
-hash of config parameters
-
-=back
-
-=head2 service_unset
+=head2 unset_config ($service_name, $config_keys)
 
 Unsets configuration value for service to restore charm defaults
 
-B<Params>
+C<service_name> - name of service
 
-=over 4
+C<config_keys> - hash of config keys to unset
 
-=item *
-
-C<service_name>
-
-name of service
-
-=item *
-
-C<config_keys>
-
-config items to unset
-
-=back
-
-=head2 service_set_charm
+=head2 set_charm ($service_name, $charm_url, $force)
 
 Sets charm url for service
 
-B<Params>
+C<service_name> - name of service
 
-=over 4
+C<charm_url> - charm location (ie. cs:precise/wordpress)
 
-=item *
-
-C<service_name>
-
-name of service
-
-=item *
-
-C<charm_url> 
-
-charm location (ie. cs:precise/wordpress)
-
-=item *
-
-C<force>
-
-(optional) for setting charm url, overrides any existing charm url already set.
-
-=back
-
-=head2 service_get
+=head2 get_service ($service_name)
 
 Returns information on charm, config, constraints, service keys.
 
-B<Params>
-
-=over 4
-
-=item *
-
 C<service_name> - name of service
 
-=back
+B<Returns> - Hash of information on service
 
-=head2 get_config
+=head2 get_config ($service_name)
 
 Get service configuration
 
-B<Params>
+C<service_name> - name of service
 
-=over 4
+B<Returns> - Hash of service configuration
 
-=item *
-
-C<service_name>
-
-name of service
-
-=back
-
-=head2 get_service_constraints
+=head2 get_service_constraints ($service_name)
 
 Returns the constraints for the given service.
 
-B<Params>
+C<service_name> - Name of service
 
-=over 4
+=head2 set_service_constraints ($service_name, $constraints)
 
-=item *
+C<service_name> - Name of service
 
-C<service_name>
+C<constraints> - Service constraints
 
-Name of service
-
-=back
-
-=head2 set_service_constraints
-
-Specifies the constraints for the given service.
-
-B<Params>
-
-=over 4
-
-=item *
-
-C<service_name>
-
-Name of service
-
-=item *
-
-C<constraints>
-
-Service constraints
-
-=back
-
-=head2 share_environment
+=head2 share_environment($users)
 
 Allows the given users access to the environment.
 
-B<Params>
-
-=over 4
-
-=item *
-
-C<users>
-
-List of users to allow access
-
-=back
-
-=head2 unshare_environment
+=head2 unshare_environment($users)
 
 Removes the given users access to the environment.
 
-B<Params>
-
-=over 4
-
-=item *
-
-C<users>
-
-List of users to remove access
-
-=back
-
-=head2 service_destroy
+=head2 service_destroy ($service_name)
 
 Destroys a service
 
-B<Params>
+C<service_name> - name of service
 
-=over 4
-
-=item *
-
-C<service_name>
-
-name of service
-
-=back
-
-=head2 service_expose
+=head2 service_expose ($service_name)
 
 Expose service
 
-B<Params>
+C<service_name> - Name of service
 
-=over 4
-
-=item *
-
-C<service_name>
-
-Name of service
-
-=back
-
-=head2 service_unexpose
+=head2 service_unexpose ($service_name)
 
 Unexpose service
 
-B<Params>
-
-=over 4
-
-=item *
-
-C<service_name>
-
-Name of service
-
-=back
+C<service_name> - Name of service
 
 =head2 service_charm_relations
 
 All possible relation names of a service
 
-B<Params>
-
-=over 4
-
-=item *
-
-C<service_name>
-
-Name of service
-
-=back
-
-=head2 add_service_units
+=head2 add_service_units ($service_name, $num_units)
 
 Adds given number of units to a service
 
-B<Params>
-
-=over 4
-
-=item *
-
-C<service_name>
-
-Name of service
-
-=item *
-
-C<num_units>
-
-Number of units to add
-
-=back
-
-=head2 add_service_unit
+=head2 add_service_unit ($service_name, $machine_spec)
 
 Add unit to specific machine
 
-B<Params>
-
-=over 4
-
-=item *
-
-C<service_name>
-
-Name of service
-
-=item *
-
-C<machine_spec>
-
-Machine to add unit to
-
-=back
-
-=head2 destroy_service_units
+=head2 destroy_service_units ($unit_names)
 
 Decreases number of units dedicated to a service
 
-B<Params>
-
-=over 4
-
-=item *
-
-C<unit_names>
-
-List of units to destroy
-
-=back
-
-=head2 resolved
+=head2 resolved ($unit_name, $retry)
 
 Clear errors on unit
 
-B<Params>
-
-=over 4
-
-=item *
-
-C<unit_name>
-
-id of unit (eg, mysql/0)
-
-=item *
-
-C<retry>
-
-Boolean to force a retry
-
-=back
+C<unit_name> - id of unit (eg, mysql/0)
+C<retry> - bool
 
 =head2 run
 
@@ -1649,44 +1164,18 @@ Not implemented yet.
 Set annotations on entity, valid types are C<service>, C<unit>,
 C<machine>, C<environment>
 
-=head2 get_annotations
+=head2 get_annotations ($entity, $entity_type)
 
 Returns annotations that have been set on the given entity.
 
-B<Params>
-
-=over 4
-
-=item *
-
-C<entity>
-
-=item *
-
-C<entity_type>
-
-=back
-
-=head2 private_address
+=head2 private_address($target)
 
 Get private address of machine or unit
 
   $self->private_address('1');  # get address of machine 1
   $self->private_address('mysql/0');  # get address of first unit of mysql
 
-B<Params>
-
-=over 4
-
-=item *
-
-C<target>
-
-Target machine
-
-=back
-
-=head2 public_address
+=head2 public_address($target)
 
 Returns the public address of the specified machine or unit. For a
 machine, target is an id not a tag.
@@ -1694,40 +1183,10 @@ machine, target is an id not a tag.
   $self->public_address('1');  # get address of machine 1
   $self->public_address('mysql/0');  # get address of first unit of mysql
 
-B<Params>
-
-=over 4
-
-=item *
-
-C<target>
-
-Target machine
-
-=back
-
-=head2 service_set_yaml
+=head2 service_set_yaml ($service, $yaml)
 
 Sets configuration options on a service given options in YAML format.
 
-B<Params>
-
-=over 4
-
-=item *
-
-C<service>
-
-Service Name
-
-=item *
-
-C<yaml>
-
-YAML formatted string of options
-
-=back
-
 =head1 AUTHOR
 
 Adam Stokes <adamjs@cpan.org>
@@ -1740,18 +1199,6 @@ This is free software, licensed under:
 
   The MIT (X11) License
 
-=head1 SEE ALSO
-
-Please see those modules/websites for more information related to this module.
-
-=over 4
-
-=item *
-
-L<Juju|Juju>
-
-=back
-
 =head1 DISCLAIMER OF WARRANTY
 
 BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
