@@ -1,6 +1,6 @@
 package Juju::RPC;
 # ABSTRACT: RPC Class
-$Juju::RPC::VERSION = '1.5.1';
+$Juju::RPC::VERSION = '1.1';
 
 use strict;
 use warnings;
@@ -8,28 +8,18 @@ use AnyEvent;
 use AnyEvent::WebSocket::Client;
 use JSON::PP;
 
-use Class::Tiny qw(conn result is_connected done), {
-    request_id => 1,
-};
+use Class::Tiny qw(conn is_connected), {request_id => 1};
 
-
-sub BUILD {
+sub create_connection {
     my $self = shift;
+    die "Already Connected."
+      if $self->is_connected and $self->is_authenticated;
     my $client = AnyEvent::WebSocket::Client->new(ssl_no_verify => 1);
-    $self->conn($client->connect($self->endpoint)->recv);
     $self->is_connected(1);
-
-    $self->conn->on(
-        each_message => sub {
-            my ($conn, $message) = @_;
-            my $body = decode_json($message->decoded_body);
-            if (defined($body->{Response})) {
-                $self->done->send($body);
-            }
-        }
-    );
+    $self->conn($client->connect($self->endpoint)->recv);
 }
 
+
 sub close {
     my $self = shift;
     $self->conn->close;
@@ -37,19 +27,23 @@ sub close {
 
 sub call {
     my ($self, $params, $cb) = @_;
-
-    $self->done(AnyEvent->condvar);
+    my $done = AnyEvent->condvar;
 
     # Increment request id
     $self->request_id($self->request_id + 1);
     $params->{RequestId} = $self->request_id;
     $self->conn->send(encode_json($params));
+    $self->conn->on(
+        each_message => sub {
+            $done->send(decode_json(pop->decoded_body)->{Response});
+        }
+    );
 
     # non-blocking
-    return $cb->($self->done->recv) if $cb;
+    return $cb->($done->recv) if $cb;
 
     # blocking
-    return $self->done->recv;
+    return $done->recv;
 }
 
 1;
@@ -66,7 +60,7 @@ Juju::RPC - RPC Class
 
 =head1 VERSION
 
-version 1.5.1
+version 1.1
 
 =head1 DESCRIPTION
 
@@ -89,45 +83,31 @@ Check if a websocket connection exists
 
 =head1 METHODS
 
+=head2 creation_connection
+
+Initiate a websocket connection and stores itself in C<conn> attribute.
+
+=head3 Returns
+
+Websocket connection
+
 =head2 close
 
 Close connection
 
 =head2 call
 
-Sends event to juju api server, this is the entrypoint for all api calls. If an
-B<error> occurs it will return a response object of:
+Sends event to juju api server
 
-  {
-    Error => 'Error message',
-    RequestId => 1,
-    Response => {}
-  }
+=head3 Takes
 
-Otherwise, successful queries will return:
+C<params> - Hash of parameters needed to query Juju API
 
-  {
-    Response => { some_successful => 'hash' }
-    RequestId => 1
-  }
+C<cb> - (optional) callback routine
 
-B<Params>
+=head3 Returns
 
-=over 4
-
-=item *
-
-C<params>
-
-Hash of request parameters
-
-=item *
-
-C<cb>
-
-(optional) callback for non-blocking operations
-
-=back
+Result of RPC Response
 
 =head1 AUTHOR
 
@@ -141,18 +121,6 @@ This is free software, licensed under:
 
   The MIT (X11) License
 
-=head1 SEE ALSO
-
-Please see those modules/websites for more information related to this module.
-
-=over 4
-
-=item *
-
-L<Juju|Juju>
-
-=back
-
 =head1 DISCLAIMER OF WARRANTY
 
 BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
